from zope.app.form.browser.add import AddView
from zope.app import zapi
from zope.security.proxy import removeSecurityProxy
from zope.app.publisher.browser import BrowserView
import datetime

class WorkLogEntryAddView(AddView):
    
    def add(self, content):
        result = self.context.add(content)
        zapi.getParents(content)[0].clockedIn = True
        return result


class ClockOut(BrowserView):
    """Class that handles alterations made after clocking out."""

    def parentURL(self):
        return zapi.absoluteURL(zapi.getParents(self.context)[0], self.request)

    def setInfo(self):
        newcontext = removeSecurityProxy(self.context) 
        newcontext.workDone = self.request.form['workDone']
        newcontext.learned = self.request.form['learned']
        newcontext.setEndTime('')
        zapi.getParents(self.context)[0].clockedIn = False
        return self.request.response.redirect(self.parentURL())


class WorkLogView(BrowserView):
    newcontext = None
    def __init__(self,context, request):
        super(WorkLogView,self).__init__(context, request)
        self.newcontext = removeSecurityProxy(context)

    def longShortCheck(self):
        """Returns true if user wants full descriptions."""
        return self.request.get(u'length') == u'long'
    
    def shorten(self, text):
        if text.__str__() != 'None':
            if text.__len__() > 100:
                return text[:99]+ '...'
            else:
                return text

    def sortWorkLogEntries(self,list):
        """Sorts worklog entries by starting time, newest entries come first."""
        list.sort(lambda x,y: cmp(y.startTime,x.startTime))
        return list

    def changeTimeZone(self, date):
        """Temporary function to change UTC time to EST"""
        date += datetime.timedelta(seconds=-14400)
        date = date.replace(tzinfo=None)
        return date
   
    def monthSelect(self):
        month = self.request.get(u'month')
        results = list(self.newcontext.values())
        newresults= []
        if month == None or month == u'all':
            newresults=results
        else:
            for item in results:
                if item.startTime.strftime('%B') == month:
                    newresults.append(item)
        if self.request.has_key(u'endmonth'):
            startdate = datetime.datetime(int(self.request.get(u'startyear')), 
                                          int(self.request.get(u'startmonth')), 
                                          int(self.request.get(u'startday')))
            enddate = datetime.datetime(int(self.request.get(u'endyear')),
                                        int(self.request.get(u'endmonth')),
                                        int(self.request.get(u'endday')),23,59,59)
            filteredresults=[]
            for item in newresults:
                #zope's datetimeutils is insufficient for time zone conversion, as far as I can tell
                #astimezone() when using datetimeutils' implementation of tzinfo gives 
                #*** ValueError: fromutc: non-None dst() result required). 
                #This is a stop-gap measure until paul tells me how to work it out.
                #This measure will only work for EST, and DOES NOT take Daylight savings time into account.
                startTime = self.changeTimeZone(item.startTime)
                if cmp(startTime, startdate) == 1 and cmp(startTime, enddate) == -1:
                    filteredresults.append(item)
            newresults = filteredresults
        newresults = self.sortWorkLogEntries(newresults)
        dictresults = []
        for item in newresults:
            unsubmittedtime= self.changeTimeZone(self.getLastSubmittedEntryTime())
            dictresults.append({'object':item,
                                'isSubmitted':cmp(self.changeTimeZone(item.startTime), unsubmittedtime) != 1,
                                'correctstarthour':item.startTime.strftime('%H'),
                                'correctstartminute':item.startTime.strftime('%M'),
                                'correctendhour':item.endTime.strftime('%H'),
                                'correctendminute':item.endTime.strftime('%M'),
                                'timeworked':self.truncateTimeDelta(item.endTime-item.startTime),
                                'shortenedworkdone':self.shorten(item.workDone),
                                'shortenedlearned':self.shorten(item.learned)})
        return dictresults

    def getLastEntryTime(self):
        return self.sortWorkLogEntries(list(self.newcontext.values()))[0].startTime.strftime('%A, %B %d, %Y at %I:%M:%S %p')

    def getLastEntryURL(self):
        return zapi.absoluteURL(self.sortWorkLogEntries(list(self.newcontext.values()))[0], self.request)

    def totalTime(self):
        #sum() does not work here- it gives an error about adding ints and timedeltas together.
        result = datetime.timedelta(0)
        timelist = [entry.endTime-entry.startTime for entry in self.newcontext.values()]
        for item in timelist:
            result += item
        return result

    def truncateTimeDelta(self, timedelta):
        """Returns a string representing a timedelta object that has no seconds or microseconds."""
        return str(timedelta)[:len(str(timedelta))-10]

    def submit(self):
        if self.request.has_key(u'submitmonth'):
            enddate = datetime.datetime(int(self.request.get(u'submityear')),
                                        int(self.request.get(u'submitmonth')),
                                        int(self.request.get(u'submitday')),
                                        int(self.request.get(u'submithour')),
                                        int(self.request.get(u'submitminute')),59)
            self.newcontext.submitted.append(enddate)
    
    def getLastSubmittedEntryTime(self):
        if len(self.newcontext.submitted) == 0:
            result = self.newcontext.values()[0].startTime
        else:
            result = self.newcontext.submitted[len(self.newcontext.submitted)-1] 
        return result

    def formatSubmitTime(self):
        return self.getLastSubmittedEntryTime().strftime('%B %d, %Y %I:%M %p')
