from zope.interface import Interface 
from zope.schema import Text, Datetime, Object, TextLine, Bool, List
from zope.app.container.interfaces import IContainer, IContained
from zope.app.container.constraints import contains, containers
class IPerson(Interface):
    """A Person object"""
    username = Text(
            title = u"Username",
            description = u"Person's log-in name",
            required = True)

    title = Text(
            title = u"Name",
            description = u"Person's full name",
            required = True)


class IPersonContainer(IContainer):
    """A container for people"""
    contains(IPerson)

class IPersonContained(IContained):
    """A Person contained in a IPersonContainer"""
    containers(IPersonContainer)

class IWorkLogEntry(Interface):
    """A work entry."""
    startTime = Datetime(
            title = u"Start Time",
            description = u"Person's start time",
            required = True)

    endTime = Datetime(
            title = u"End Time",
            description = u"Person's end time",
            required = False)

    workDone = Text(
            title = u"Work Done",
            description = u"What s/he did",
            required = False)

    learned = Text(
            title = u"What you learned",
            description = u"What s/he learned",
            required = False)

class IWorkLog(IContainer):
    """A container for work entries."""

    submitted = List(
            title = u'Submitted',
            description = u'List of dates that timesheets were submitted on.',
            required=False)

    person = Object(
           title = u'Personthingy',
           description = u'Owner of WorkLog',
           schema = IPerson,
           required =True)

    """person = TextLine(
            title = u'Username',
            description = u'Owner of Worklog',
            required = True)"""

    title = TextLine(
           title = u"Work Log Title",
           description = u"Title of Work Log",
           required = False)

    clockedIn = Bool(
           title = u"Clocked In",
           description = u"Whether or not the person is clocked in.",
           required = False)

    contains(IWorkLogEntry)

class IWorkLogEntryContained(IContained):
    """A WorkLogEntry contained in a WorkLogEntryContainer"""
    containers(IWorkLog)

class IWorkLogContainer(IContainer):
    """A container for worklog."""

    contains(IWorkLog)

class IWorkLogContained(IContained):
    """A WorkLog contained in WorkLogContainer."""

    containers(IWorkLogContainer)
